#   -*- coding: utf-8 -*-
#
#   This file is part of SKALE.py
#
#   Copyright (C) 2019-Present SKALE Labs
#
#   SKALE.py is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   SKALE.py is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with SKALE.py.  If not, see <https://www.gnu.org/licenses/>.

from skale.contracts.base_contract import BaseContract, transaction_method


class Linker(BaseContract):
    @transaction_method
    def connect_schain(
            self,
            schain_name: str,
            community_locker_address: str,
            token_manager_eth_address: str,
            token_manager_erc20_address: str,
            token_manager_erc721_address: str
    ):
        return self.contract.functions.connectSchain(
            schain_name,
            [
                community_locker_address,
                token_manager_eth_address,
                token_manager_erc20_address,
                token_manager_erc721_address
            ]
        )
